﻿using NMEA_Transport;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace FusionLinkLite
{
    public partial class Form1 : Form
    {
        ITransport nmea = null;
        bool connected = false;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            nmea = new NMEATransport();
            nmea.PacketReceived = PacketReceived;
        }

        private void btnProbe_Click(object sender, EventArgs e)
        {
            btnConnect.Enabled = false;
            pnlCommands.Enabled = false;
            lvDevices.Items.Clear();
            connected = false;
            nmea.BroadcastIsoRequest(NMEACommon.N2K_PROD_INFO_ID);
        }

        private void lvDevices_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lvDevices.SelectedItems.Count > 0)
            {
                NMEA_Device dev = lvDevices.SelectedItems[0].Tag as NMEA_Device;

                nmea.SetAudioAddress(dev.Address);

                lblDeviceName.Text = dev.Model;
                btnConnect.Text = "Connect";
                btnConnect.Enabled = true;
            }
        }

        private void btnConnect_Click(object sender, EventArgs e)
        {
            connected = true;
            pnlCommands.Enabled = true;
        }

        void AddToDevicesList(NMEA_Device new_device)
        {
            foreach (ListViewItem item in lvDevices.Items)
            {
                NMEA_Device dev = item.Tag as NMEA_Device;

                if (dev.SerialNumber == new_device.SerialNumber)
                {
                    return;
                }
            }

            ListViewItem new_item = new ListViewItem();
            new_item.Tag = new_device;
            new_item.Text = new_device.Address.ToString();
            new_item.SubItems.Add(new_device.Model);
            new_item.SubItems.Add(new_device.ModelVersion);
            new_item.SubItems.Add(new_device.SWVersion);

            lvDevices.Items.Add(new_item);
        }

        void PacketReceived(NMEA_Packet packet)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke((Action)delegate()
                {
                    ProcessPacketReceived(packet);
                });
            }
            else
            {
                ProcessPacketReceived(packet);
            }
        }

        void ProcessPacketReceived(NMEA_Packet packet)
        {
            if (packet.PGN == NMEACommon.N2K_PROD_INFO_ID)
            {
                NMEA_Device new_device = new NMEA_Device(packet);
                AddToDevicesList(new_device);
            }
            else
            {
//                AudioPacket command = AudioPacket.ParsePacket(packet);

//                ProcessAudioPacket(command);
            }
        }

        private void btnNextSource_Click(object sender, EventArgs e)
        {
            SendRequest(FusionLiteCommands.NEXT_SOURCE);
        }

        private void btnPreviousSource_Click(object sender, EventArgs e)
        {
            SendRequest(FusionLiteCommands.PREVIOUS_SOURCE);
        }

        private void btnVolumeUp_Click(object sender, EventArgs e)
        {
            SendRequest(FusionLiteCommands.VOLUME_UP);
        }

        private void btnVolumeDown_Click(object sender, EventArgs e)
        {
            SendRequest(FusionLiteCommands.VOLUME_DOWN);
        }

        private void btnSkipCommand_Click(object sender, EventArgs e)
        {
            SendRequest(FusionLiteCommands.SKIP_COMMAND);
        }

        private void btnPreviousCommand_Click(object sender, EventArgs e)
        {
            SendRequest(FusionLiteCommands.PREVIOUS_COMMAND);
        }

        private void btnMuteToggle_Click(object sender, EventArgs e)
        {
            SendRequest(FusionLiteCommands.MUTE_TOGGLE);
        }

        private void btnPlayToggle_Click(object sender, EventArgs e)
        {
            SendRequest(FusionLiteCommands.PLAY_TOGGLE);
        }

        private void SendRequest(FusionLiteCommands fusionLiteCommands)
        {
            if(nmea != null)
            {
                nmea.SendPacket(ConvertRequest((ushort)fusionLiteCommands));
            }
        }

        private byte[] ConvertRequest(ushort cmd)
        {
            List<byte> result = new List<byte>();

            result.AddRange(UInt16ToBytes(cmd));

            return result.ToArray();
        }

        protected byte[] UInt16ToBytes(UInt16 value)
        {
            if (BitConverter.IsLittleEndian)
            {
                return new byte[2]
                {
                    (byte)(value & 0xFF),
                    (byte)((value >> 8) & 0xFF)
                };
            }
            else
            {
                return new byte[2]
                {
                    (byte)((value >> 8) & 0xFF),
                    (byte)(value & 0xFF)
                };
            }
        }
    }
}
