﻿using System.Runtime.InteropServices;

using u8 = System.Byte;
using u16 = System.UInt16;
using u32 = System.UInt32;
using s32 = System.Int32;
using size_t = System.UInt32;
using System;
using System.Reflection;

namespace ActisenseComms
{
    namespace API
    {
        public partial class Const 
        {
            public const uint USE_DEFAULTS_U32 = 0xFFFFFFFE;
        };

        public enum stream_t {
            /* Recommended enumerations for new software designs */
            CmdStreamBST = 0,
            CmdStreamN183,
            CmdStreamN2K,
            /* Normally this value only returned where an error has occurred - e.g. handle was invalid */
            CmdStreamUnknown = 0xFF
        };

        public enum PGNEnable_t {
	        /* Disable a PGN */
	        DISABLE_PGN = 0,
	        /* Enable a PGN */
	        ENABLE_PGN = 1	
        }; 

        /* Delete an entire 'PGN Enable List' ID definitions */
        public enum PGNEnableList_t {
	        /* CAN Rx 'PGN Enable List' */
	        ENABLE_LIST_RX = 0,
	        /* CAN Tx 'PGN Enable List' */
	        ENABLE_LIST_TX = 1,
	        /* Both CAN Rx and CAN Tx 'PGN Enable List' */
	        ENABLE_LIST_RXTX = 2
        };

        public enum PGNMask_t : uint {
	        /* For PDU1 type messages: PDU Specific = Destination Address */
	        /* Match PGN and Exact Source Address Mask.
	           Mask includes the full 26 bits (ignoring 3-bit priority):
	           R, DP, PDU Format, PDU Specific and Source Address bits */
	        MATCH_PGN_ADDRESS_MASK = 0x03FFFFFF,
	        /* Match PGN Mask (allows all Source Addresses).
	           Mask includes the top 18 bits (ignoring 3-bit priority):
	           R, DP, PDU Format & PDU Specific bits. Source Address = 'do not care' */
	        MATCH_PGN_MASK         = 0x03FFFF00,
	        /* Match PDU Format Mask (allows all PDU Specifics & Source Addresses).
	           Mask includes the top 10 bits (ignoring 3-bit priority):
	           R, DP & PDU Format. PDU Specific & Source Address = 'do not care' */
	        MATCH_PDU_FORMAT_MASK  = 0x03FF0000,
	        /* Request the use of CAN Library default Mask */
	        USE_DEFAULT_PGN_MASK =  Const.USE_DEFAULTS_U32
	    };

        public class AComms
        {
            public enum ePortEnumStatus {
                ACPES_Available,
                /* Port is unavailable for some other reason than already being open */
                ACPES_Unavailable,
                ACPES_Open,
                /* Port scan has not yet been performed - A method will be made 
                   available to scan through all ports and discover whether they are 
                   open, closed or unavailable.  This was in the original API, but the 
                   method is slow for some types so was removed to be made a separate 
                   function so that enumeration can be perfomed quickly */
                ACPES_Unknown
            };
          
            [StructLayout(LayoutKind.Sequential)]
            public unsafe struct sPortEnum {
                public const int Count = 256;
	            /* Number of Comm ports found by the enumeration function */
	            public u32 Size;
	            /* List of Comm port numbers found (of length 'Size') */
                [MarshalAs(UnmanagedType.ByValArray, SizeConst = Count)]
                public u32[] Ports;				
	            /* Status of Comm ports found by the enumeration function */
                [MarshalAs(UnmanagedType.ByValArray, SizeConst = Count)]
                public ePortEnumStatus[] PortStatus;
            };

            /*************************************************************************
            Function 	: Create

            Refer to Actisense Comms SDK User Manual for details.

            __in_out	out int Handle - pointer (used to return created handle required
                          by all access functions that operate on this resource)  
            __out		int Error code (negative value), if handle could not be created
                          (Refer to ARLErrorCodes header for details of these codes)
            **************************************************************************/ 
            [DllImport("ActisenseComms.dll", EntryPoint="ACommsCreate", CallingConvention=CallingConvention.StdCall, CharSet=CharSet.Ansi)]
            public static extern int Create(out int Handle);
            
            /*************************************************************************
            Function 	: Destroy

            Refer to Actisense Comms SDK User Manual for details.

            __in        int Handle to resource to destroy (returned by ACommsCreate)
            __out		int Error code, if resource recovery failed (Refer to ARLErrorCodes)
            **************************************************************************/ 
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDestroy", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int Destroy(int Handler);
            
            /*************************************************************************
            Function 	: DestroyAll

            Refer to Actisense Comms SDK User Manual for details.

            __in		void
            __out		int Error code, if resource recovery failed (Refer to ARLErrorCodes)
            **************************************************************************/ 
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDestroyAll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int DestroyAll();
            
            /*************************************************************************
            Function 	: Open

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in		int Port Number to link Comms object to
            __in		int Baud rate to open port with
            __out		int Error code, if open port failed (Refer to ARLErrorCodes)
            **************************************************************************/ 
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsOpen", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int Open(int Handle, int PortNumber, int Baudrate);
            
            /*************************************************************************
            Function 	: Close

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __out		int Error code, if close port failed (Refer to ARLErrorCodes)
            **************************************************************************/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsClose", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int Close(int Handler);
            
            /*************************************************************************
            Function 	: GetPortNumber

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms oject (returned by ACommsCreate)
            __in		int out PortNumber - (used to return Port number)
            __out		int Error code, if Get Port Number failed (Refer to ARLErrorCodes)
            **************************************************************************/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsGetPortNumber", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetPortNumber(int Handler, out int PortNumber);
            
            /*************************************************************************
            Function 			: GetPortBaudrate

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in		int out Baudrate - (used to return Baud rate)
            __out		int Error code, if Get Baudrate failed (Refer to ARLErrorCodes)
            **************************************************************************/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsGetPortBaudrate", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetPortBaudrate(int Handle, out int Baudrate);
            
            /*************************************************************************
            Function 	: GetRxLoading
            Function 	: GetTxLoading

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __inout		int out The loading factor on a port - returns percentage value x 1000 
			              (0.000 - 100.000 %) in the passed in reference to int
            __ret		int Error code
            **************************************************************************/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsGetRxLoading", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetRxLoading(int Handle, out int Loading);
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsGetTxLoading", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetTxLoading(int Handle, out int Loading);
            
            /*************************************************************************
            Function 	: EnumerateSerialPorts

            Refer to Actisense Comms SDK User Manual for details.

            __in		out sPortEnum Port Enumeration - (to user defined structure
                          that will be filled in with the active enumerated ports list 
                          currently available to the system) 
            __out		int Error code, if Enumeration failed (Refer to ARLErrorCodes)
            **************************************************************************/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsEnumerateSerialPorts", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int EnumerateSerialPorts(out sPortEnum PortEnum);
            
            /*************************************************************************
            Function 	: EnumerateSerialPortsGetName

            Refer to Actisense Comms SDK User Manual for details.

            __in		u32 Port Number of the Comms port whose name is required
            __out		string Friendly Name - reference to a text string of maximum 
                          length 256 characters (including null terminator)
			              Note : This string storage will only persist until this 
                          function called again, so calling function should not use this
                          pointer directly, but instead copy this string as soon as the
                          function returns it.
            **************************************************************************/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsEnumerateSerialPortsGetName", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern string EnumerateSerialPortsGetName(u32 PortNum);
        }

        public class ACommand
        {
            /* API Command Setup Functions */
            /* --------------------------------------------------------------------------
             Function    : GetStream

             Refer to Actisense Comms SDK User Manual for details.

             __in		int Handle to Comms object (returned by AComms.Create)
             __in       out stream_t returns current Stream API commands are using
             __out		int Error code (negative value), if given stream deemed invalid
                          (Refer to ARLErrorCodes header for details of these codes)
             --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_GetStream", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetStream(int Handle, out stream_t Stream);
            
            /*--------------------------------------------------------------------------
            Function    : SetStream

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        stream_t New Stream to send future API commands to
            __out		int Error code (negative value), if given stream deemed invalid
                          (Refer to ARLErrorCodes header for details of these codes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_SetStream", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int SetStream(int Handle, stream_t Stream);
            
            /*--------------------------------------------------------------------------
            Function    : GetN2KAddress

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        out u8 returns current NMEA 2000 Address API commands are
				          using (if stream is also set to NMEA 2000)
            __out		int Error code, if Address deemed invalid (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_GetN2KAddress", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetN2KAddress(int Handle, out u8 N2KAddress);
            
            /*--------------------------------------------------------------------------
            Function    : SetN2KAddress

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        u8  New NMEA 2000 Address to route future API command to
                          (if stream is also set to NMEA 2000)
            __out		int Error code, if Address deemed invalid (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_SetN2KAddress", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int SetN2KAddress(int Handle, u8 N2KAddress);
            
            /* API Command Functions */
            /*************************************************************************
            TARGET UNIT COMMANDS:
            THESE COMMANDS ARE SENT OVER THE CURRENT COMMAND STREAM TO A TARGET 
            (local or remote) ACTISENSE DEVICE
            **************************************************************************
            --------------------------------------------------------------------------
            Function    : Reboot
            Decode      : Decode.GetTag
            DataType    : ddConfirmSoftwareReset

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_Reboot", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int Reboot(int Handle);
            
            /*--------------------------------------------------------------------------
            Function    : ReInitMainApp
            Decode      : Decode.GetTag
            DataType    : ddHardwareInfo

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_ReInitMainApp", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int ReInitMainApp(int Handle);
            
            /*--------------------------------------------------------------------------
            Function    : CommitToEEPROM
            Decode      : Decode.GetTag
            DataType    : ddCommitToEEPROM

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_CommitToEEPROM", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int CommitToEEPROM(int Handle);
            
            /*--------------------------------------------------------------------------
            Function    : CommitToFlash
            Decode      : Decode.GetTag
            DataType    : ddCommitToFlash

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_CommitToFlash", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int CommitToFlash(int Handle);
            
            /*************************************************************************
            GENRAL INFORMATION AND DEVICE SETTING COMMANDS: 
            THESE COMMANDS HANDLE BASIC SETTINGS OF THE TARGET DEVICE
            **************************************************************************
            --------------------------------------------------------------------------
            Function    : GetHardwareInfo
            Decode      : Decode.GetHardwareInfo
            DataType    : ddHardwareInfo

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_GetHardwareInfo", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetHardwareInfo(int Handle);
            
            /*--------------------------------------------------------------------------
            Function    : GetOperatingMode
            Decode      : Decode.GetOperatingMode
            DataType    : ddOperatingMode

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_GetOperatingMode", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetOperatingMode(int Handle);
            
            /*--------------------------------------------------------------------------
            Function    : SetOperatingMode
            Decode      : Decode.GetOperatingMode
            DataType    : ddOperationingMode

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        u16 OperatingMode contains new Operationing mode
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_SetOperatingMode", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int SetOperatingMode(int Handle, u16 OperatingMode);
            
            /*--------------------------------------------------------------------------
            Function    : GetHardwareBaudCodes
            Decode      : Decode.GetHardwareBaudCodes
            DataType    : ddOperationingMode

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
           --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_GetHardwareBaudCodes", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetHardwareBaudCodes(int Handle);
            
            /*--------------------------------------------------------------------------
            Function    : SetHardwareBaudCodes
            Decode      : Decode.GetHardwareBaudCodes
            DataType    : ddHardwareBaud

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        int Number of Baud codes (0 if command is simple request for data)
            __in        u8[] BaudCodes contains new Baud codes. reference is NULL if command 
                          is simple request for data). Each member is set to the required 
                          Baud rate code defined in the ARLBaudCode header
            __out       int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_SetHardwareBaudCodes", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int SetHardwareBaudCodes(int Handle, int nBaudCodes, u8[] BaudCodes);
            
            /*--------------------------------------------------------------------------
            Function    : GetPortBaudCodes
            Decode      : Decode.GetPortBaudCodes
            DataType    : ddHardwareBaud

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __out       int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_GetPortBaudCodes", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetPortBaudCodes(int Handle);
            
            /*--------------------------------------------------------------------------
            Function    : SetPortBaudCodes
            Decode      : Decode.GetPortBaudCodes
            DataType    : ddPortBaudCfg

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        int Number of Baud codes (0 if command is simple request for data)
            __in        u8[] BaudCodes - reference to array containing new Baud codes.
                          (reference is NULL if command is simple request for data)
				          Each member is set to the required Baud rate code defined
                          in the ARLBaudCode header
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_SetPortBaudCodes", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int SetPortBaudCodes(int Handle, int nBaudCodes, u8[] BaudCodes);
            
            /*--------------------------------------------------------------------------
            Function    : GetPortPCodes
            Decode      : Decode.GetPortPCodes
            DataType    : ddHardwareBaud

            Refer to Actisense Comms SDK User Manual for details.

            __in	    int Handle to Comms object (returned by ACommsCreate)
            __out       int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_GetPortPCodes", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetPortPCodes(int Handle);
            
            /*--------------------------------------------------------------------------
            Function    : SetPortPCodes
            Decode      : Decode.GetPortPCodes
            DataType    : ddPortPCodeCfg

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        int Number of P code enables (0 if command is simple request for data)
            __in        u8  PCodes	- reference to array containing new P code enables. (reference is NULL if command is simple request for data)
				          Each member is 0 for disable, or 1 for enable
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_SetPortPCodes", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int SetPortPCodes(int Handle, int nPCodes, u8[] PCodes);
            
            /*--------------------------------------------------------------------------
            Function    : SetPortDupDelete
            Decode      : Decode.GetPortDupDelete
            DataType    : ddPortDupDelete

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        int Number of Port duplicate delete settings 
                          (0 if command is simple request for data)
            __in        u8  PortDupDelete - reference to array containing new duplicate
				          delete enables (reference is NULL if command is simple request for data)
				          Each member is 0 for disable, or 1 for enable
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_GetPortDupDelete", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetPortDupDelete(int Handle);

            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_SetPortDupDelete", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int SetPortDupDelete(int Handle, int nPortDupDelete, u8[] PortDupDelete);
            
            /*--------------------------------------------------------------------------
            Function    : GetTotalTime
            Decode      : Decode.GetTotalTime
            DataType    : ddTotalTime

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_GetTotalTime", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetTotalTime(int Handle);
            
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_SetTotalTime", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int SetTotalTime(int Handle, u32 TotalTime, u32 Passkey);
            
            /*************************************************************************
            NMEA 2000 COMMANDS: 
            THESE COMMANDS HANDLE NMEA 2000 / CAN BUS RELATED SETTINGS
            FOR THE TARGET DEVICE, SO AR EONLY RELEVANT TO ACTISENSE 
            DEVICES  THAT SUPPORT NMEA 2000 
            **************************************************************************
            --------------------------------------------------------------------------
            Function    : GetProductInfoN2K
            Decode      : Decode.GetProductInfoN2K
            DataType    : ddProductInfoN2K

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_GetProductInfoN2K", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetProductInfoN2K(int Handle);
            
            /*--------------------------------------------------------------------------
            Function    : GetCanConfig
            Decode      : Decode.GetCanConfig
            DataType    : ddCANConfig

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_GetCanConfig", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetCanConfig(int Handle);
            
            /*--------------------------------------------------------------------------
            Function    : SetCanConfig
            Decode      : Decode.GetCanConfig
            DataType    : ddCANConfig

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        int Preferred Address 0 -252
            __in        int New System Instance - Use defined value DONOT_CHANGE_U8
                          (0xFF) if no change to system instance field is required
            __in        int New Device Instance - Use defined value DONOT_CHANGE_U8
                          (0xFF) if no change to device instance field is required
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_SetCanConfig", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int SetCanConfig(int Handle, int Preferred, int SystemInstance, int DeviceInstance);
            
            /*--------------------------------------------------------------------------
            Function    : SetCanInfoField1
            Function    : SetCanInfoField2
            Function    : SetCanInfoField3
            Function    : GetCanInfoField1
            Function    : GetCanInfoField2
            Function    : GetCanInfoField3

            Decode      : Decode.GetCanInfo
            DataType    : ddCANInfoField 1 / 2 / 3

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        string InfoString - containing the requested new CAN information text string. 
                          ("Decode.Get" Functions do not have this parameter)
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_SetCanInfoField1", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int SetCanInfoField1(int Handle, string InfoString);
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_SetCanInfoField2", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int SetCanInfoField2(int Handle, string InfoString);
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_SetCanInfoField3", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int SetCanInfoField3(int Handle, string InfoString);
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_GetCanInfoField1", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetCanInfoField1(int Handle);
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_GetCanInfoField2", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetCanInfoField2(int Handle);
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_GetCanInfoField3", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetCanInfoField3(int Handle);
            
            /*--------------------------------------------------------------------------
            Function    : SetRxPGN
            Function    : SetRxPGNEx
            Decode      : Decode.GetRxPGN
            DataType    : ddEnableRxPGN

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        u32 NMEA 2000 PGN Number being set up
            __in        u32 Enable (1) or Disable (0) status
            __in        PGNmask_t Enumerated type Mask value
                          (PGNmask_t defined as ActisenseComms.API.PGNEnable_t).
                          #defines from library: USE_DEFAULT_PGN_MASK to specify the
                          library default set in the Gateway.
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_SetRxPGN", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int SetRxPGN(int Handle, u32 PGN, PGNEnable_t Enable);
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_SetRxPGNEx", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int SetRxPGNEx(int Handle, u32 PGN, PGNEnable_t Enable, PGNMask_t Mask);
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_GetRxPGN", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetRxPGN(int Handle, u32 PGN);
            
            /*--------------------------------------------------------------------------
            Function    : SetTxPGN 
            Function    : SetTxPGNEx
            Function    : GetTxPGN
            DataType    : ddEnableTxPGN

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        u32 NMEA 2000 PGN Number being set up
            __in        u32 Enable (1) or Disable (0) status
            __in        u32 Desired transmission rate of data in milliseconds - 
                          this is time period between successive transmissions of this
                          PGN. Device programmed to have a minimum setting for this, 
                          and will respond with an error code if the setting is too low.
				          #defines from library: USE_DEFAULTS_U32 for "Use default"
            __in        u32 For repeated PGNs, this setting is the Timeout for the data
                          - the gateway must receive new data from the PC within this
                          time period, or the PGN will stop being sent.
				          Const.USE_DEFAULTS_U32 for "Use default"
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_SetTxPGN", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int SetTxPGN(int Handle, u32 PGN, PGNEnable_t Enable);
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_SetTxPGNEx", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int SetTxPGNEx(int Handle, u32 PGN, PGNEnable_t Enable, u32 Rate, u32 Timeout);      
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_GetTxPGN", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetTxPGN(int Handle, u32 PGN);
            
            /*--------------------------------------------------------------------------
            Function    : GetRxPGNList
            Decode      : Decode.GetRxPGNList
            DataType    : ddRxPGNEnableList

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_GetRxPGNList", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetRxPGNList(int Handle);
            
            /*--------------------------------------------------------------------------
            Function    : GetTxPGNList
            Decode      : Decode.GetTxPGNList
            DataType    : ddTxPGNEnableList

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_GetTxPGNList", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetTxPGNList(int Handle);
            
            /*--------------------------------------------------------------------------
            Function    : ClearPGNList
            Function    : ClearRxPGNList
            Function    : ClearTxPGNList
            Decode      : Decode.GetTag
            DataType    : ddDeletePGNEnableList

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        PGNEnableList_t	List ID - one of:
				          ENABLE_LIST_RX, ENABLE_LIST_TX or ENABLE_LIST_RXTX (delete both lists)
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_ClearPGNList", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int ClearPGNList(int Handle, PGNEnableList_t ListID);
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_ClearRxPGNList", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int ClearRxPGNList(int Handle);
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_ClearTxPGNList", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int ClearTxPGNList(int Handle);
            
            /*--------------------------------------------------------------------------
            Function    : ActivatePGNEnableLists 
            Decode      : Decode.GetActivatePGNEnableListResponse
            DataType    : ddActivatePGNEnableLists

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_ActivatePGNEnableLists", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int ActivatePGNEnableLists(int Handle);
            
            /*--------------------------------------------------------------------------
            Function    : SetDefaultPGNEnableList
            Decode      : Decode.GetTag
            DataType    : ddDefaultPGNEnableList

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        PGNEnableList_t ListID - one of : ENABLE_LIST_RX, ENABLE_LIST_TX, or ENABLE_LIST_RXTX (default both lists)
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_SetDefaultPGNEnableList", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int SetDefaultPGNEnableList(int Handle, PGNEnableList_t ListID);
            
            /*--------------------------------------------------------------------------
            Function    : GetParamsPGNEnableLists
            Decode      : Decode.GetParamsPGNEnableLists
            DataType    : ddParamsPGNEnableLists

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_GetParamsPGNEnableLists", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetParamsPGNEnableLists(int Handle);
        }

        [StructLayout(LayoutKind.Sequential, Pack = 1)]
        public struct sDecodeTag {
            public u32 ResponseTime;			/* Command response time in milliseconds -
									               can be used to test the comms channel's
									               performance. For regular messages, like
									               System Status, this becomes the time
									               between messages instead */
            public u32 PCTickCount;				/* Tick count at time of decode */
            public s32 Error;					/* Error code for message (Refer to ARLErrorCode.h) */
            public u32 Serial;					/* Unique ARL Serial number of the unit
									               that returned this message */
            public u32 ModelID;					/* ARL Model ID number (unique to each device type) */
            public DecodeData_t DataType;		/* The data type this tag is attached to */
            public stream_t SrcStream;			/* Source Stream that this message was received from */
            public u32 SrcAddress;				/* Source Address that this message came from 	
									               (0 if Source Stream set to BST / 'local') */
        }

        public enum DecodeDetail_t {
	        /* A description function gives a brief description of the supplied variable */
	        ddNameBrief,
	        /* A description function gives a full description of the supplied variable (if available) */
	        ddNameFull,
	        /* A description function gives a detailed description of the supplied variable (if available) */
	        ddNameDetailed,
	        /* Maximum value of DecodeDetail_t */
	        ddNameMax
        };

        public class Decode
        {
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_GetAge", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern s32 ACommsDecode_GetAge(out sDecodeTag DecodeTag);
           
            /*--------------------------------------------------------------------------
            Function    : GetDataTypeName

            Refer to Actisense Comms SDK User Manual for details.

            __in        DataType - enumerated DecodeData_t data type code
            __in        DecodeDetail - enumerated DecodeDetail_t.
            __out       string - the name of the data type as a string
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_GetDataTypeName", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern string GetDataTypeName(DecodeData_t DataType, DecodeDetail_t DecodeDetail);
            
            /*--------------------------------------------------------------------------
            Function    : GetUARTBaudCodeName
            Function    : GetCANBaudCodeName

            Refer to Actisense Comms SDK User Manual for details.

            __in        u32 Code number representing enumerated Code
            __in        DecodeDetail - enumerated DecodeDetail_t, requesting required 
                          detail of returned string required - see "API_Decode_Datatypes.h"
            __out       string -  string which describes the enumerated code value
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_GetUARTBaudCodeName", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern string GetUARTBaudCodeName(u32 Code, DecodeDetail_t DecodeDetail);
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_GetCANBaudCodeName", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern string GetCANBaudCodeName(u32 Code, DecodeDetail_t DecodeDetail);
            
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_GetModelIDName", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern string GetModelIDName(u32 Code, DecodeDetail_t DecodeDetail);

            public delegate void Callback(ref object UserCallbackData, DecodeData_t DecodeData, DecodeReason_t DecodeReason);

            /*--------------------------------------------------------------------------
            Function    : SetCallback

            Refer to Actisense Comms SDK User Manual for details.

            Important:  No windows should be created by the callback function, as the 
                        wait loop within the Comms thread does not process windows 
                        messages to reduce overhead; if the callback function creates
                        a window, it could cause a system deadlock due to the 
                        unprocessed message queue.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        DataType_t Enumerated DecodeData_t Callback data type being registered
            __in        void Callback(ref object UserCallbackData, DecodeData_t DecodeData, DecodeReason_t DecodeReason
                          Refer to documentation below
            __in        int User defined Callback data for user specific functionality
            __out		int Error code (negative value), if Set operation deemed invalid
                          (Refer to ARLErrorCodes header for details of these codes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_SetCallback", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int SetCallback(int Handle, DecodeData_t DataType, Callback callback, ref object UserCallbackData);
            
            /*--------------------------------------------------------------------------
            Function    : SetCallbackGroup

            Refer to '_SetCallback' description above and the Actisense Comms SDK 
            User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        DataType_t Pointer to an array of enumerated DecodeData_t 
                          Callback data types being registered
            __in        int Size of data type array (number of data types that will 
                          trigger the calling of the given callback function)
            __in        void Callback(ref object UserCallbackData, DecodeData_t DecodeData, DecodeReason_t DecodeReason
                          Refer to documentation above
            __in        int User defined Callback data for user specific functionality
            __out		int Error code (negative value), if Set operation deemed invalid
                          (Refer to ARLErrorCodes header for details of these codes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_SetCallbackGroup", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int SetCallbackGroup(int Handle, out DecodeData_t DataType, int DataTypeSize, Callback callback, ref object UserCallbackData);

            /*--------------------------------------------------------------------------
            Function    : GetTag

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        out sDecodeTag - a decode tag structure to hold the
                          responding information
            __in        DecodeData_t - data type whose tag data is to be returned the
				          enumerated type DecodeData_t
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_GetTag", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetTag(int Handle, out sDecodeTag Tag, DecodeData_t DataType);
            
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_GetTagAndData", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetTagAndData(int Handle, out u8[] Message, DecodeData_t DataType, u32 MessageMaxSize);
            
            /*--------------------------------------------------------------------------
            Function    : GetHardwareInfo
            Command     : Command.GetHardwareInfo
            DataType    : ddHardwareInfo

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        out sHardwareInfo HardInfo - The API user must create this structure
                          and pass in a pointer to it. The API will then fill out the
                          structure with the information decoded from the device response
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_GetHardwareInfo", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetHardwareInfo(int Handle, out sDecodeHardwareInfo HardInfo);
            
            /*--------------------------------------------------------------------------
            Function    : GetOperatingMode
            Command     : Command.GetOperatingMode
            Command     : Command.SetOperatingMode
            DataType    : ddOperatingMode

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        sDecodeArray_u8: out BaudCodes The API user must create this structure
                          and pass in a pointer to it. The API will then fill out the
                          structure with the information decoded from the device response
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_GetOperatingMode", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetOperatingMode(int Handle, out sDecodeOperatingMode OperatingMode);
            
            /*--------------------------------------------------------------------------
            Function    : GetHardwareBaudCodes
            Command     : Command.GetHardwareBaudCodes
            Command     : Command.SetHardwareBaudCodes
            DataType    : ddHardwareBaudCfg

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        sDecodeArray_u8: out BaudCodes The API user must create this structure
                          and pass in a reference to it. The API will then fill out the
                          structure with the information decoded from the device response
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_GetHardwareBaudCodes", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetHardwareBaudCodes(int Handle, out sDecodePortBaudCodes BaudCodes);
            
            /*--------------------------------------------------------------------------
            Function    : GetPortBaudCodes
            Command     : Command.GetPortBaudCodes
            Command     : Command.SetPortBaudCodes
            DataType    : ddPortBaudCfg

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        sDecodeArray_u8: out BaudCodes The API user must create this structure
                          and pass in a pointer to it. The API will then fill out the
                          structure with the information decoded from the device response
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_GetPortBaudCodes", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetPortBaudCodes(int Handle, out sDecodePortBaudCodes BaudCodes);
           
            /*--------------------------------------------------------------------------
            Function    : GetPortPCodes
            Command     : Command.GetPortPCodes
            Command     : Command.SetPortPCodes
            DataType    : ddPortPCodeCfg

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        sDecodeArray_u8: out PCodes The API user must create this structure 
                          and pass in a pointer to it. The API will then fill out the
                          structure with the information decoded from the device response
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_GetPortPCodes", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetPortPCodes(int Handle, out sDecodeArray_u8 PCodes);
            
            /*--------------------------------------------------------------------------
            Function    : GetPortDupDelete
            Command     : Command.GetPortDupDelete
            Command     : Command.SetPortDupDelete
            DataType    : ddPortDupDelete

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        sDecodeArray_u8: out DupDelete The API user must create this structure
                          and pass in a pointer to it. The API will then fill out the
                          structure with the information decoded from the device response
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_GetPortDupDelete", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetPortDupDelete(int Handle, out sDecodeArray_u8 DupDelete);
            
            /*--------------------------------------------------------------------------
            Function    : GetTotalTime
            Command     : Command.GetTotalTime
            Command     : Command.SetTotalTime
            DataType    : ddTotalTime

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        sDecodeTotalTime : out TotalTime The API user must create this 
                          structure and pass in a reference to it. The API will then fill
                          out structure with the information decoded from the device response
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_GetTotalTime", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetTotalTime(int Handle, out sDecodeTotalTime TotalTime);
            
            /*--------------------------------------------------------------------------
            Function    : GetProductInfoN2K
            Command     : Command.GetProductInfoN2K
            DataType    : ddProductInfoN2K

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        sDecodeProductInfoN2K : ProductInfoN2K The API user must create
                          this structure and pass in a reference to it. The API will then
                          fill out the structure with the information decoded from the 
                          device response
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_GetProductInfoN2K", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetProductInfoN2K(int Handle, out sDecodeProductInfoN2K ProductInfoN2K);
            
            /*--------------------------------------------------------------------------
            Function    : GetCanConfig
            Command     : Command.GetCanConfig
            Command     : Command.SetCanConfig
            DataType    : ddCanConfig

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        out sDecodeCanConfig  CanConfig The API user must create this structure
                          and pass in a reference to it. The API will then fill out the
                          structure with the information decoded from the device response
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_GetCanConfig", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetCanConfig(int Handle, out sDecodeCanConfig CanConfig);
            
            /*--------------------------------------------------------------------------
            Function    : GetCanInfoField 1 / 2 / 3
            Command     : Command.GetCanInfoField 1 / 2 / 3
            Command     : Command.SetCanInfoField 1 / 2 / 3
            DataType    : ddCanInfoField 1 / 2 / 3

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        out sDecodeCanInfoField CanInfoField The API user must create this
                          structure and pass in a reference to it. The API will then fill
                          out structure with the information decoded from the device response
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_GetCanInfoField1", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetCanInfoField1(int Handle, out sDecodeCanInfoField CanInfoField1);
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_GetCanInfoField2", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetCanInfoField2(int Handle, out sDecodeCanInfoField CanInfoField2);
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_GetCanInfoField3", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetCanInfoField3(int Handle, out sDecodeCanInfoField CanInfoField3);
            
            /*--------------------------------------------------------------------------
            Function    : GetRxPGN
            Command     : Command.GetRxPGN
            Command     : Command.SetRxPGN
            Command     : Command.SetRxPGNEx
            DataType    : ddEnableRxPGN

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        out sDecodeRxPGN DecodeRxPGN The API user must create this structure
                          and pass in a referemce to it. The API will then fill out the
                          structure with the information decoded from the device response
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_GetRxPGN", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetRxPGN(int Handle, out sDecodeRxPGN DecodeRxPGN);
            
            /*--------------------------------------------------------------------------
            Function    : GetTxPGN
            Command     : Command.GetTxPGN
            Command     : Command.SetTxPGN
            Command     : Command.SetTxPGNEx
            DataType    : ddEnableTxPGN

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        out sDecodeTxPGN DecodeTxPGN The API user must create this structure
                          and pass in a reference to it. The API will then fill out the
                          structure with the information decoded from the device response
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_GetTxPGN", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetTxPGN(int Handle, out sDecodeTxPGN DecodeTxPGN);
            
            /*--------------------------------------------------------------------------
            Function    : GetRxPGNList
            Command     : Command.GetRxPGNList
            DataType    : ddRxPGNEnableList

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        out sDecodeRxPGNList DecodeRxPGNList The API user must create this
                          structure and pass in a reference to it. The API will then fill
                          out structure with the information decoded from the device response
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_GetRxPGNList", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetRxPGNList(int Handle, out sDecodeRxPGNList DecodeRxPGNList);
            
            /*--------------------------------------------------------------------------
            Function    : GetTxPGNList
            Command     : Command.GetTxPGNList
            DataType    : ddTxPGNEnableList

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        out sDecodeTxPGNList DecodeTxPGNList The API user must create this
                            structure and pass in a reference to it. The API will then fill
                            out the structure with the information decoded from the device
                            response
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_GetTxPGNList", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetTxPGNList(int Handle, out sDecodeTxPGNList DecodeTxPGNList);
           
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_GetParamsPGNEnableLists", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetParamsPGNEnableLists(int Handle, out sDecodePGNEnableListStatus PGNListStatus);
            
            /*--------------------------------------------------------------------------
            Function    : GetStartupStatus
            Command     : None - this message is sent once only when device starts up
            DataType    : ddStartupStatus

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        out sDecodeStartupStatus StartupStatus The API user must create this
                         structure and pass in a reference to it. The API will then fill 
                         out the structure with the information decoded from the device response
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_GetStartupStatus", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetStartupStatus(int Handle, out sDecodeStartupStatus StartupStatus);
           
            /*--------------------------------------------------------------------------
            Function    : GetSystemStatus
            Command     : None - this message is sent once per second (if P-codes enabled)
            DataType    : pSystemStatus

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        out sDecodeSystemStatus SystemStatus The API user must create this
                          structure and pass in a reference to it. The API will then fill 
                          out the structure with the information decoded from the device response
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_GetSystemStatus", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetSystemStatus(int Handle, out sDecodeSystemStatus SystemStatus);
            
            /*--------------------------------------------------------------------------
            Function    : GetDbgTimeProfiler
            Command     : None - this message is sent once per second (if P-codes enabled & debug firmware)
            DataType    : ddDebugTimeProfiler

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        out sDbgTimeProfiler TimeProfile The API user must create this
                          structure and pass in a reference to it. The API will then fill 
                          out the structure with the information decoded from the device response
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsDecode_GetDbgTimeProfiler", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetDbgTimeProfiler(int Handle, out sDbgTimeProfiler TimeProfile);
        }

        public delegate void Callback(Missing UserData);

        public class BST
        {
            public unsafe struct sMsg {
                const int cBSTType1MaxLen = (223 + 11);
                public u32 Timestamp;
                public u32 ID;
                public u32 Length;
                public fixed u8 Data[cBSTType1MaxLen];
            };

            /*--------------------------------------------------------------------------
            Function : Write

            Refer to Actisense Comms SDK User Manual for details.

            __in	    int Handle to Comms object (returned by ACommsCreate)
            __in          out BST.sMsg msg - reference to BST message to transmit 
                          in BST.sMsg structure format
            __out	    int Error code (negative value), if operation deemed invalid
                          (Refer to ARLErrorCodes header for details of these codes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsBST_Write", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int Write(int Handle, out sMsg msg);
            
            /*--------------------------------------------------------------------------
            Function    : Read

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __out       out BST.sMsg msg - reference to empty BST message structure
                          where received BST data will get stored in BST.sMsg structure format 
                          BST.sMsg msg - BST message data is copied into this supplied structure
            __out       int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsBST_Read", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int Read(int Handle, out sMsg msg);
            
            /*--------------------------------------------------------------------------
            Function    : GetRxQSize

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __out       out size_t BufferSize - reference to size_t in which to store size
                          of buffer
                        out size_t BufferSize - Size of buffer in BST Messages, 
                          0 if an error was detected
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsBST_GetRxQSize", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetRxQSize(int Handle, out size_t BufferSize);
            
            /*--------------------------------------------------------------------------
            Function : FlushRx

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __out	    int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsBST_FlushRx", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int FlushRx(int Handle);
            
            /*--------------------------------------------------------------------------
            Function : FlushTx

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/ 
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsBST_FlushTx", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int FlushTx(int Handle);
            
            /*--------------------------------------------------------------------------
            Function : SetRxCallback

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        delegate void Callback (Missing UserData) - delegate to user callback method
            __in        Missing UserData - placeholder
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsBST_SetRxCallback", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int SetRxCallback(int Handle, Callback callback, Missing UserData);
        }

        public class NMEA0183
        {
            [StructLayout(LayoutKind.Sequential)]
            public unsafe struct sMsg {
                const int NMEA0183_MAXLEN = 300;
                public u32 Timestamp;
                public u32 Size;
                public fixed char Data[NMEA0183_MAXLEN];
            };

            /*--------------------------------------------------------------------------
            Function    : Write

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        out NMEA183.sMsg msg - reference to NMEA 0183 message to transmit 
                          in NMEA183.sMsg structure format
            __out		int Error code (negative value), if operation deemed invalid
                          (Refer to ARLErrorCodes header for details of these codes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsN183_Write", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int Write(int Handle, out sMsg msg);
            
            /*--------------------------------------------------------------------------
            Function    : Read

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __out       out NMEA183.sMsg msg - reference to empty NMEA 0183 message structure
                          where received NMEA0183 data will get stored in NMEA183.sMsg structure format out NMEA0183.sMsg msg -
                          NMEA0183 message data is copied int this supplied structure
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsN183_Read", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int Read(int Handle, out sMsg msg);
            
            /*--------------------------------------------------------------------------
            Function    : GetRxQSize

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __out       out size_t BufferSize - reference to size_t in which to store size
                          of buffer
                        out size_t BufferSize - Size of buffer in NMEA 0183 Messages, 
                          0 if an error was detected
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsN183_GetRxQSize", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetRxQSize(int Handle, out size_t BufferSize);
            
            /*--------------------------------------------------------------------------
            Function : FlushRx

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/           
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsN183_FlushRx", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int FlushRx(int Handle);
            
            /*--------------------------------------------------------------------------
            Function : FlushTx

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/           
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsN183_FlushTx", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int FlushTx(int Handle);
            
            /*--------------------------------------------------------------------------
            Function : SetRxCallback

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        delegate void Callback (Missing UserData) - delegate to user callback method
            __in        Missing UserData - placeholder
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsN183_SetRxCallback", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int SetRxCallback(int Handle, Callback callback, Missing UserData);
        }

        public class NMEA2K
        {
            [StructLayout(LayoutKind.Sequential)]
            public unsafe struct sMsg {
                const int N2K_MAXLEN_TP = 1785;
                const int N2K_MAXLEN_FP = 223;
                const int N2K_MAXLEN_SF = 8;
                public u32 Timestamp;
                public u32 PGN;
                public u8  Priority;
                public u8  SrcAddr;
                public u8  DestAddr;
                public u32 Length;
                public fixed u8 Data[N2K_MAXLEN_TP];
            }
            
            /*--------------------------------------------------------------------------
            Function : Write

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        out NMEA2K.sMsg msg - pointer to NMEA 2000 message to transmit in 
                          NMEA2K.sMsg structure format
            __out		int Error code (negative value), if operation deemed invalid
                          (Refer to ARLErrorCodes header for details of these codes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsN2K_Write", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int Write(int Handle, out sMsg msg);
            
            /*--------------------------------------------------------------------------
            Function : Read

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        out NMEA2K.sMsg msg - reference to empty NMEA 2000 message structure
				          where received NMEA 2000 data will get stored in 
                          NMEA2K.sMsg structure format
            __out       out NMEA2K.sMsg	msg - NMEA 2000 message data is copied into this 
                          supplied structure
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/            
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsN2K_Read", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int Read(int Handle, out sMsg msg);
            
            /*--------------------------------------------------------------------------
            Function : GetRxQSize

            Refer to Actisense Comms SDK User Manual for details.

            __in    int Handle to Comms object (returned by ACommsCreate)
            __in    out size_t BufferSize - reference to size_t in which to store size of buffer
            __out   out size_t BufferSize - size of buffer in NMEA 2000 Messages, 0 if an error was detected
            __out   int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/            
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsN2K_GetRxQSize", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int GetRxQSize(int Handle, out size_t BufferSize);
            
            /*--------------------------------------------------------------------------
            Function : FlushRx

            Refer to Actisense Comms SDK User Manual for details.

            __in    int Handle to Comms object (returned by ACommsCreate)
            __out   int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/           
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsN2K_FlushRx", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int FlushRx(int Handle);
            
            /*--------------------------------------------------------------------------
            Function    : FlushTx

            Refer to Actisense Comms SDK User Manual for details.

            __in    int Handle to Comms object (returned by ACommsCreate)
            __out   int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsN2K_FlushTx", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int FlushTx(int Handle);
            
            /*--------------------------------------------------------------------------
            Function : SetRxCallback

            Refer to Actisense Comms SDK User Manual for details.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        void Callback(Missing UserData) - delegate to user callback method
            __in        Missing UserData - placeholder
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsN2K_SetRxCallback", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int SetRxCallback(int Handle, Callback callback, Missing UserData);
        }

        public class Log
        {
            public class Mask
            {
                public const u32 COMMSLOG_EBL_RX = 0x0001;
	            public const u32 COMMSLOG_EBL_TX = 0x0002;
            };
            /*-------------------------------------------------------------------------
            Function    :  int Enable

            Refer to Actisense Comms SDK User Manual for details.

            __in        int Handle : Current Handle for the Actisense comms resource
            __in        string FileStem : Stem for the filename to use
            __in        u32 EnableMask : Required logging for this comms resource
            __out		int Error code (negative value), if given stream deemed invalid
                          (Refer to ARLErrorCodes header for details of these codes)
            -------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsLog_Enable", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int Enable (int Handle, string FileStem, u32 EnableMask);
        };

        public class Program
        {
            /*--------------------------------------------------------------------------
            Function    : SetSector
            Decode      : Decode.GetTag
            DataType    : ddProgramSetSector

            Initialises a sector of Flash ROM (prior to erasing, writing or reading).

            This sub-command must be sent prior to any other 'Program' sub-command - 
            as it sets up the required Flash ROM sector that all subsequent operations
            will use as a base. If the given 'Sector ID' is not recognised, then the
            target will respond with the appropriate ARL Error code and no further
            'Program' operations will be possible.

            Once the sector has been initialised, any subsequent Write and Read requests
            will use that sectors base address to add the requested address offset to.

            Refer to "ARLFlashSectorCodes.h" for the full enumerated Flash ROM Sector
            ID codes list. These sectors codes uniquely identify each and every Flash
            ROM reprogrammable area in every ARL product. This allows the sector to be
            reprogrammed without the PC software requesting such an operation ever 
            having to know the actual address of the Flash ROM sector. This greatly
            increases the security and flexibility of the reprogramming operation.

            Refer to "BEMProgramProtocol.h" for a full BYTE-by-BYTE breakdown of the
            various commands and responses.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        u16 SectorID
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_ProgramSetSector", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]   
            public static extern int SetSector (int Handle, u16 SectorID);
           
            /*--------------------------------------------------------------------------
            Function    : ACommsCommand_ProgramEraseSector
            Decode      : Decode.GetTag
            DataType    : ddProgramErase

            Erase the current sector of Flash ROM (prior to reprogramming).

            The actual sector that this operation is performed on is dependent on how
            the target has been instructed to understand the 'Sector ID'. This allows
            each target to operate within its own Flash ROM limits whilst still using
            the same group of reprogramming interface sub-commands.

            Refer to "BEMProgramProtocol.h" for a full BYTE-by-BYTE breakdown of the
            various commands and responses.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
                --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_ProgramEraseSector", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int EraseSector(int Handle);
            
            /* --------------------------------------------------------------------------
            Function    : ProgramWriteVerifyBlock
            Decode      : Decode.GetTag
            DataType    : ddProgramWriteVerify

            The Write block size that the microcontroller can cope with (due to its 
            RAM limitations on handling the receiption of these reprogramming messages),
            was returned to the PC software in the 'Set Sector' response message.

            Writes (and then Verifies) a block data bytes (up to the maximum size that
            the target set) to the requested Flash ROM offset address inside the
            currently set Sector.

            Refer to "BEMProgramProtocol.h" for a full BYTE-by-BYTE breakdown of the
            various commands and responses.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        u32 Address block - address
            __in        u32 DataSize - block size
            __in        u8[] Data - block data
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_ProgramWriteVerifyBlock", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int WriteVerifyBlock (int Handle, u32 AddressOffset, u32 DataSize, u8[] Data);
            
            /*--------------------------------------------------------------------------
            Function    : ProgramFinaliseBlock
            Decode      : Decode.GetTag
            DataType    : ddProgramFinalise

            Finalise/Complete the Flash ROM reprogramming operation by giving hardware
            the CRC and size of the entire Data Block transferred. These values will be
            used by the hardware to generate its own CRC of the modified Flash ROM area,
            so it can be matched against the given CRC and the corresponding response
            returned to the PC software.

            Refer to "BEMProgramProtocol.h" for a full BYTE-by-BYTE breakdown of the
            various commands and responses.

            __in		int Handle to Comms object (returned by ACommsCreate)
            __in        u32 BlockCRC  
            __in        u32 BlockSize
            __out		int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_ProgramFinaliseBlock", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int FinaliseBlock(int Handle, u32 BlockCRC, u32 BlockSize);
            
            /*--------------------------------------------------------------------------
            Function    : ReadBlock
            Decode      : Decode.GetTag
            DataType    : ddProgramRead

            The Read block size that the microcontroller can cope with (due to its 
            RAM limitations on handling the transmission of these programming messages),
            was returned to the PC software in the 'Set Sector' response message.

            Reads a block data bytes (up to the maximum size that the target set) from
            the requested Flash ROM offset address inside the currently set Sector.

            Refer to "BEMProgramProtocol.h" for a full BYTE-by-BYTE breakdown of the
            various commands and responses.

            __in	int Handle to Comms object (returned by ACommsCreate)
            __in    u32 Address, 
            __in    u8 BlockSize
            __out	int Error code, if operation failed (Refer to ARLErrorCodes)
            --------------------------------------------------------------------------*/
            [DllImport("ActisenseComms.dll", EntryPoint = "ACommsCommand_ProgramReadBlock", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
            public static extern int ReadBlock(int Handle, u32 Address, u32 BlockSize);
        };
    }  
}
