﻿
namespace ActisenseComms
{
    namespace API
    {
        public enum DecodeData_t {
            /*----------------------------------------------------------------------
             ARL BST/BEM Embedded Commands :
             All commands sent from the PC using BEM-ID code BST_BEMP_MSG_PG (0xA1),
             will use BEM-ID code BST_BEMP_MSG_GP (0xA0) for all required responses
             (ACKnowledge / NACKnowldege or Information), plus the BEM sub-command.
            -----------------------------------------------------------------------*/
            
            /* BST-BEM Re-Boot Hardware command/response */
            /* Causes hardware to return to its Bootloader (before returning back to Main Application after 
               set time - normally 1 second). Used to catch Bootloader prior to performing a Main Application 
               reprogram operation */
            ddReBootPG                  = 0x11FE,
            
            
            /*---  Action sub-commands 0x00-0x0F  ---------------------------------*/
            /*----------------------------------------------------------------------
              Action commands used to trigger major operating mode changes on the
              attached device. Only to be used if the consequences are understood.
            -----------------------------------------------------------------------*/
            
            /* BST-BEM Re-Initialisation of Main Application command response */
            /* Response to 'Re-initialise' command. Standard BEM header only (no BEM data) */
            ddReInitMainApp             = 0xA000,
            
            /* BST-BEM Commit (changed configuration) To EEPROM command response */
            /* Response to 'Commit to EEPROM' command. Standard BEM header only (no BEM data) */
            ddCommitToEEPROM            = 0xA001,
            
            /* BST-BEM Commit (changed configuration) To Flash ROM command response */
            /* Response to 'Commit to Flash' command. Standard BEM header only (no BEM data) */
            ddCommitToFlash             = 0xA002,
            
            
            /*---  Request sub-commands 0x10-0xEF  --------------------------------*/
            /*----------------------------------------------------------------------
              Request commands either change settings on attached device, or query
              information/settings stored inside the attached device.
              Commands sent with no data/parameters are trigger requests for the
              attached device to send back the requested information.
              Commands with optional data/parameters not included will request that
              the attached device uses its built-in default values to complete the
              required request.
            -----------------------------------------------------------------------*/
            
            /* BST-BEM Request Hardware Information command response */
            /* Response to 'Request Hardware Information' command. Contains Bootloader and Main Application 
               details, the Actisense Hardware version, Total Operating Time (in seconds), Model Sub-ID 
               and Operating Mode */
            ddHardwareInfo              = 0xA010,
            
            /* BST-BEM Request/Change Operating Mode command response */
            /* Response to 'Request/Change Operating Mode' command. Contains 'Operating Mode' device is 
               using. Indicates the result of a 'Operating Mode' change command, or the current 'Operating 
               Mode' if requested */
            ddOperatingMode             = 0xA011,
            
            /* BST-BEM Request/Change Port Baud rates command response */
            /* Response to 'Request/Change Port Baud rates' command. Contains 'Port Baud rates' for each 
               port the device supports. Indicates the currently set EEPROM 'Baud rates' configuration */
            ddPortBaudCfg               = 0xA012,
            
            /* BST-BEM Request/Change Port P-Code enables command response */
            /* Response to 'Request/Change Port P-codes' command. Contains 'Port P-code enable' for each 
               port the device supports. Indicates the currently set EEPROM 'P-codes enables' configuration 
               */
            ddPortPCodeCfg              = 0xA013,
            
            /* BST-BEM Request/Change Port 'Duplicate Deletion' enables command response */
            /* Response to 'Request/Change Port Duplicate Delete' command. Contains 'Port 'Duplicate Delete 
               enable' for each port the device supports. Indicates the currently set EEPROM 'Duplicate 
               Delete enables' configuration */
            ddPortDupDelete             = 0xA014,
            
            /* BST-BEM Request/Change Total (Operating) Time command response */
            /* Response to 'Request/Change Total (Operating) Time' command. Contains 'Total (Operating) 
               Time' of the device (in seconds). Indicates the result of a 'Total (Operating) Time' change 
               command, or the current 'Total (Operating) Time' if requested */
            ddTotalTime                 = 0xA015,
            
            /* BST-BEM Request/Change Hardware Baud rates command response */
            /* Response to 'Request/Change Hardware Baud rates' command. Contains 'Hardware Baud rates' 
               for each hardware port the device supports. Indicates the current hardware 'Baud rates' in 
               use */
            ddHardwareBaud              = 0xA016,
            
            /* BST-BEM Request NMEA 2000 Product Information command response */
            /* Response to 'Request NMEA 2000 Product Information' command. Contains the defined at compile/program 
               time set of 'CAN Product Info' of any NMEA 2000 device. Non-NMEA 2000 devices will return 
               an appropriate error code */
            ddProductInfoN2K            = 0xA041,
            
            /* BST-BEM Request/Change (NMEA 2000) CAN Config command response */
            /* Response to 'Request/Change NMEA 2000 CAN Config' command. Contains the CAN Address Claim 
               details of the device. Indicates the result of the 'CAN Config' change command, or the current 
               'CAN Config' if requested */
            ddCANConfig                 = 0xA042,
            
            /* BST-BEM Request/Change (NMEA 2000) CAN Information Field 1 command response */
            /* Response to 'Request/Change NMEA 2000 CAN Info Field 1' command. Contains the 'CAN Info Field 
               1' of the device. Indicates the result of the 'CAN Info Field 1' change command, or the current 
               'CAN Info Field 1' if requested */
            ddCANInfoField1             = 0xA043,
            
            /* BST-BEM Request/Change (NMEA 2000) CAN Information Field 2 command response */
            /* Response to 'Request/Change NMEA 2000 CAN Info Field 2' command. Contains the 'CAN Info Field 
               2' of the device. Indicates the result of the 'CAN Info Field 2' change command, or the current 
               'CAN Info Field 2' if requested */
            ddCANInfoField2             = 0xA044,
            
            /* BST-BEM Request (NMEA 2000) CAN Information Field 3 command response */
            /* Response to 'Request/Change NMEA 2000 CAN Info Field 3' command. Contains the 'CAN Info Field 
               3' of the device. Indicates the result of the 'CAN Info Field 3' change command, or the current 
               'CAN Info Field 3' if requested */
            ddCANInfoField3             = 0xA045,
            
            /* BST-BEM Request/Change an Rx PGN Enable status command response */
            /* Response to 'Request/Change Rx PGN Enable status' command. Contains either the requested 
               or changed 'Rx PGN Enable status' details currently held in the device */
            ddEnableRxPGN               = 0xA046,
            
            /* BST-BEM Request/Change an Tx PGN Enable status command response */
            /* Response to 'Request/Change Tx PGN Enable status' command. Contains either the requested 
               or changed 'Tx PGN Enable status' details currently held in the device */
            ddEnableTxPGN               = 0xA047,
            
            /* BST-BEM Request Rx PGN Enable List command response */
            /* Response to 'Request Rx PGN Enable List' command. Contains the requested 'Rx PGN Enable List' 
               currently held in the device, seperated in to two messages. Message 1 contains the PGN List 
               */
            ddRxPGNEnableList           = 0xA048,
            
            /* BST-BEM Request Tx PGN Enable List command response */
            /* Response to the 'Request Tx PGN Enable List' command. Contains the requested 'Tx PGN Enable 
               List' currently held in the device, seperated in to four messages. Message 1 contains the 
               PGN List */
            ddTxPGNEnableList           = 0xA049,
            
            /* BST-BEM Request to Delete a PGN Enable List command response */
            /* Response to the 'Request Deletion of the entire Rx or Tx PGN Enable List' from the Session 
               (RAM). Contains the 'List ID' of the list requested (and actioned if valid) to confirm which 
               list has been deleted */
            ddDeletePGNEnableList       = 0xA04A,
            
            /* BST-BEM Request to Activate a PGN Enable List command response */
            /* Response to the 'Request Activation of current PGN Enable List'. Contains the currently 'in 
               use' and maximum capacity values for each PGN Enable List ( Rx 'Real', Rx 'Virtual' & Tx 
               'Virtual'), plus the 'Hardware Synchronisation' status flag (that indicates if the Hardware 
               settings were setup using the current Enable List values or the previous ones) */
            ddActivatePGNEnableLists    = 0xA04B,
            
            /* BST-BEM Request to Set a Default PGN Enable List command response */
            /* Response to the 'Request revert Rx and/or Tx PGN Enable List(s) back to factory defaults'. 
               Contains the 'List ID' of the list requested (and actioned if valid) to confirm which list 
               has been set back to default values */
            ddDefaultPGNEnableList      = 0xA04C,
            
            /* BST-BEM Request PGN Enable List Parameters command response */
            /* Response to the 'Request PGN Enable List Parameters'. Contains the currently 'in use' and 
               maximum capacity values for each PGN Enable List ( Rx 'Real', Rx 'Virtual' & Tx 'Virtual'), 
               plus the 'Hardware Synchronisation' status flag (that indicates if the Hardware settings 
               were setup using the current Enable List values or the previous ones) */
            ddParamsPGNEnableLists      = 0xA04D,
            
            
            /*---  Report sub-commands 0xF0-0xFC  ---------------------------------*/
            /*----------------------------------------------------------------------
              Report data is sent by the attached device whenever a trigger event
              occurs. This can be during startup, when an error's detected or at
              a specific interval (e.g. system status once per second). The user
              cannot request these messages.
            -----------------------------------------------------------------------*/
            
            /* BST-BEM 'Startup Status' information report */
            /* Full Startup Status details - this message is never Requested, it is always Broadcast every 
               time the device powers on. */
            ddStartupStatus             = 0xA0F0,
            
            /* BST-BEM 'Error Status' information report */
            /* Full Error Report details - this message is never Requested, it is always Broadcast every 
               time an error is detected. This message can optionally be preceeded with a Hardware Information 
               message (if required) so the receiving device has all the required information to decode 
               the report correctly */
            ddErrorReport               = 0xA0F1,
            
            /* BST-BEM 'System Status' information report */
            /* Full System Status details - this message is never Requested, it is always Broadcast every 
               second from the connected device, when enabled for transmit. The Data payload varies depending 
               upon the product's requirements */
            ddSystemStatus              = 0xA0F2,
            
            /* BST-BEM Confirm Software Reset command response */
            /* Device response to a \"Software reset request\". Could be added in future as alternate way 
               to get device to software reset. Allows PC software to know when device is about to enter 
               it's Bootloader, so that it can know when to start sending Bootloader specific commands */
            ddConfirmSoftwareReset      = 0xA0F3,
            
            /* BST-BEM Negative Acknowledgement report */
            /* Negative Response from the device based on previous requests of it. Not normally used */
            ddNegativeAcknowledge       = 0xA0F4,
            
            
            /*---  Action sub-commands 0x00-0x0F  ---------------------------------*/
            /*----------------------------------------------------------------------
              Action commands used to trigger major operating mode changes on the
              attached device. Only to be used if the consequences are understood.
            -----------------------------------------------------------------------*/
            
            /* BST-BEM Re-Initialisation of Main Application command */
            /* Forces device to re-initialise itself. Can be used to force the use of newly modifed EEPROM/Flash 
               configuration settings, or as a quick way to revert back to the default/current EEPROM/Flash 
               configuration settings */
            ddReInitMainAppPG           = 0xA100,
            
            /* BST-BEM Commit (changed configuration) To EEPROM command */
            /* Triggers the storing of any modified configuration settings that are saved to EEPROM - making 
               them persistent to a reset. This new configuration will then be used at all future initialisations 
               (until changed again) */
            ddCommitToEEPROMPG          = 0xA101,
            
            /* BST-BEM Commit (changed configuration) To Flash ROM command */
            /* Triggers the storing of any modified configuration settings that are saved to Flash - making 
               them persistent to a reset. This new configuration will then be used at all future initialisations 
               (until changed again) */
            ddCommitToFlashPG           = 0xA102,
            
            
            /*---  Request sub-commands 0x10-0xEF  --------------------------------*/
            /*----------------------------------------------------------------------
              Request commands either change settings on attached device, or query
              information/settings stored inside the attached device.
              Commands sent with no data/parameters are trigger requests for the
              attached device to send back the requested information.
              Commands with optional data/parameters not included will request that
              the attached device uses its built-in default values to complete the
              required request.
            -----------------------------------------------------------------------*/
            
            /* BST-BEM Request Hardware Information command */
            /* Requests Hardware Information from the device. The optional 'ID / address' (Model ID) can 
               be added to selectively retrieve Hardware Information from a device beyond the first device 
               that receives this request */
            ddHardwareInfoPG            = 0xA110,
            
            /* BST-BEM Request/Change Operating Mode command */
            /* Requests or Changes the 'Operating Mode' of a device. Allows a single ARL Model ID to have 
               multiple 'Operating Modes'. Each mode can change the operating features of a device, making 
               it more flexible. Change option requires a 32-bit Passkey */
            ddOperatingModePG           = 0xA111,
            
            /* BST-BEM Request/Change Port Baud rates command */
            /* Requests or Changes a devices Port Baud rates. Number of Baud codes sent will be device dependant. 
               Device will respond with its current settings to a no-parameter request. Automatically commits 
               new values to EEPROM to make changes persistant. Does not change hardware port settings (until 
               a re-initialisation occurs) */
            ddPortBaudCfgPG             = 0xA112,
            
            /* BST-BEM Request/Change Port P-Code enables command */
            /* Enables or Disables Actisense 'P-code' transmissions on a Comms port. Number of 'P-code's 
               sent will be device dependant. Device will respond with its current settings to a no-parameter 
               request. Must follow with the 'EEPROM Commit' command to make these changes persistant */
            ddPortPCodeCfgPG            = 0xA113,
            
            /* BST-BEM Request/Change Port 'Duplicate Deletion' enables command */
            /* Enables or Disables 'Duplicate Deletion' on a Comms port. Number of 'Duplicate Deletion's 
               sent will be device dependant. Device will respond with its current settings to a no-parameter 
               request. Must follow with the 'EEPROM Commit' command to make these changes persistant */
            ddPortDupDeletePG           = 0xA114,
            
            /* BST-BEM Request/Change Total (Operating) Time command */
            /* Request or Change a devices 'Total Operating Time' (in seconds since new). Settings stored 
               in EEPROM, automatically updated every minute (or immediately if an EEPROM Commit Request 
               is sent). Change option only used in exceptional situations (in a factory environment), and 
               requires a 32-bit Passkey */
            ddTotalTimePG               = 0xA115,
            
            /* BST-BEM Request/Change Hardware Baud rates command */
            /* Requests or Changes a devices Hardware Baud rates. Number of Baud codes sent will be device 
               dependant. Device will respond with its current settings to a no-parameter request. Does 
               not change EEPROM Port Baud values, so changes are NOT persistant */
            ddHardwareBaudPG            = 0xA116,
            
            /* BST-BEM Request NMEA 2000 Product Information command */
            /* Requests 'NMEA 2000 Product Information' from the device. This information is identical to 
               that obtained via the NMEA 2000 bus connection when the PGN 126996 is requested */
            ddProductInfoN2KPG          = 0xA141,
            
            /* BST-BEM Request/Change (NMEA 2000) CAN Config command */
            /* Requests or Changes (NMEA 2000) CAN Configuration settings. Settings stored in EEPROM for 
               retrieval at startup. Can only change CAN Name parameters for System & Device instance fields. 
               Automatically commits new values to EEPROM to make changes persistant. Also triggers a reclaim 
               of the CAN Address using the new values */
            ddCANConfigPG               = 0xA142,
            
            /* BST-BEM Request/Change (NMEA 2000) CAN Information Field 1 command */
            /* Requests or Changes (NMEA 2000) 'CAN Config Information Field 1' settings. Installation text 
               field 1 can be changed as required. Automatically commits new values to EEPROM to make changes 
               persistant */
            ddCANInfoField1PG           = 0xA143,
            
            /* BST-BEM Request/Change (NMEA 2000) CAN Information Field 2 command */
            /* Requests or Changes (NMEA 2000) 'CAN Config Information Field 2' settings. Installation text 
               field 2 can be changed as required. Automatically commits new values to EEPROM to make changes 
               persistant */
            ddCANInfoField2PG           = 0xA144,
            
            /* BST-BEM Request (NMEA 2000) CAN Information Field 3 command */
            /* Requests (NMEA 2000) 'CAN Config Information Field 3' settings. Manufacturer Info (field 
               3) - cannot be change from the Manufacturer details. Automatically commits new values to 
               EEPROM to make changes persistant */
            ddCANInfoField3PG           = 0xA145,
            
            /* BST-BEM Request/Change an Rx PGN Enable status command */
            /* Adds or Deletes the requested PGN from the current 'Rx Enable List'. Default values can be 
               requested for any optional parameters not given. Once all changes made to the list, must 
               follow with 'EEPROM Commit' command if required to make changes persistent, and 'Activate 
               List' command to re-init hardware */
            ddEnableRxPGNPG             = 0xA146,
            
            /* BST-BEM Request/Change an Tx PGN Enable status command */
            /* Adds or Deletes the requested PGN from the current 'Tx Enable List'. Default values can be 
               requested for any optional parameters not given. Once all changes made to the list, must 
               follow with 'EEPROM Commit' command if required to make changes persistent, and 'Activate 
               List' command to re-init hardware */
            ddEnableTxPGNPG             = 0xA147,
            
            /* BST-BEM Request Rx PGN Enable List command */
            /* Requests the full Rx PGN Enable List from the device. Details all Rx PGNs that are currently 
               enabled in the device (and the associated Rx Mask for each PGN) */
            ddRxPGNEnableListPG         = 0xA148,
            
            /* BST-BEM Request Tx PGN Enable List command */
            /* Requests the full Tx PGN Enable List from the device. Details all Rx PGNs that are currently 
               enabled in the device (and the associated Rx Mask for each PGN) */
            ddTxPGNEnableListPG         = 0xA149,
            
            /* BST-BEM Request to Delete a PGN Enable List command */
            /* Request to 'Delete the entire Rx or Tx PGN Enable List' from the Session (RAM) values. When 
               the new replacement list has been created, must finish with 'Activate List' command. If a 
               delete of the EEPROM values is also required, must follow with 'EEPROM Commit' command to 
               make this delete operation persistent */
            ddDeletePGNEnableListPG     = 0xA14A,
            
            /* BST-BEM Request to Activate a PGN Enable List command */
            /* Request to 'Activate all PGNs on the newly created Enable List' by re-initialising the hardware 
               to use the new PGN settings. These Session (RAM) value changes will cause a small pause in 
               CAN traffic and so must be used sparingly */
            ddActivatePGNEnableListsPG  = 0xA14B,
            
            /* BST-BEM Request to Set a Default PGN Enable List command */
            /* Request to 'Revert Rx and/or Tx PGN Enable List(s) back to factory Default settings' and 
               then automatically activate the new List(s) by re-initialising the hardware. Restores to 
               both EEPROM and Session (RAM) values in one command. Will cause a full re-initialisation 
               of device, so must be used sparingly */
            ddDefaultPGNEnableListPG    = 0xA14C,
            
            /* BST-BEM Request PGN Enable List Parameters command */
            /* Requests the 'PGN Enable List Parameters'. Useful to request whenever changes have been made 
               to either of the PGN Enable Lists - to update the number of 'PGN slots' in use and the number 
               still available */
            ddParamsPGNEnableListsPG    = 0xA14D,
            
            
            /*---  BEM Debug Protocol Status --------------------------------------*/
            /*----------------------------------------------------------------------
              All Debug messages are sent from the Product using BEM Sub-ID code
              BST_BEMP_DBG_GP (0xA2)
            -----------------------------------------------------------------------*/
            
            /*---  Non-specific sub-commands 0x00-0x1F  ---------------------------*/
            /*----------------------------------------------------------------------
              Common debug codes - should not be product or software specific
            -----------------------------------------------------------------------*/
            
            /* BST-BEM 'Text string' Debug report */
            /* Debug 'Text' - this message is never Requested, it is always Broadcast whenever certain conditions 
               are met in the target,when enabled for transmit. A simple null terminated ASCII text string 
               that varies depending upon the product's requirements. Strings length is variable */
            ddDebugText                 = 0xA200,
            
            /* BST-BEM 'U32 Value' Debug report */
            /* Debug 'Value' - this message is never Requested, it is always Broadcast whenever certain 
               conditions are met in the target,when enabled for transmit. A 32-bit unsigned integer value, 
               optionally followed by a null terminated ASCII text string (to describe what the value is) 
               that varies depending upon the product's requirements. Strings descriptor length is variable 
               */
            ddDebugValueU32             = 0xA201,
            
            /* BST-BEM 'S32 Value' Debug report */
            /* Debug 'Value' - this message is never Requested, it is always Broadcast whenever certain 
               conditions are met in the target,when enabled for transmit. A 32-bit signed integer value, 
               optionally followed by a null terminated ASCII text string (to describe what the value is) 
               that varies depending upon the product's requirements. Strings descriptor length is variable 
               */
            ddDebugValueS32             = 0xA202,
            
            
            /*---  Specific sub-commands 0x20-0x3F  -------------------------------*/
            /*----------------------------------------------------------------------
              Project/Hardware type specific program codes - specific to a
              particular products, product group types or hardware types.
              No need to group codes into areas: new codes assigned as necessary
              wherever the standard codes are not workable for a new use.
            -----------------------------------------------------------------------*/
            
            /* BST-BEM 'Time Profiler' Debug report */
            /* Time Profiler Performance Debug Status details - this message is never Requested, it is always 
               Broadcast every second from the connected device, when enabled for transmit. The Data varies 
               depending upon the product's requirements */
            ddDebugTimeProfiler         = 0xA220,
            
            
            /*---  BEM Program Protocol Commands ----------------------------------*/
            /*----------------------------------------------------------------------
              All commands sent from the PC using BEM Sub-ID code BST_BEMP_PRG_PG (0xA4),
              will use BEM sub-ID code BST_BEMP_MSG_GP (0xA3) for all required responses
              (ACKnowledge / NACKnowldege or Information), plus the BEM ID command.
            -----------------------------------------------------------------------*/
            
            /*---  Non-specific sub-commands 0x00-0x1F  ---------------------------*/
            /*----------------------------------------------------------------------
              Common debug codes - should not be product or software specific
            -----------------------------------------------------------------------*/
            
            /* BST-BEM Set Sector Program command response */
            /* Reponse to the 'Request to Set the Sector of Flash ROM'. Contains the curent 'Sector ID', 
               the 'Sector size' (in bytes), the 'Maxmium Transfer data Block size' (in bytes), and the 
               Codec ID to use for any transfers. These values must be used to cotrol all following transfers 
               to this Flash ROM sector */
            ddProgramSetSector          = 0xA300,
            
            /* BST-BEM Erase Program command response */
            /* Reponse to the 'Request to Erase current Sector of Flash ROM'. Contains the 'Sector ID' of 
               the sector erased (if successful) */
            ddProgramErase              = 0xA301,
            
            /* BST-BEM Write & Verify Program command response */
            /* Reponse to the 'Request to Write (and Verify) a Block of Flash ROM data'. Contains the 'Sector 
               ID' and size of the Block written to (if successful). Can be used to verify if the expected 
               sector and Block size were actioned */
            ddProgramWriteVerify        = 0xA302,
            
            /* BST-BEM Finalise Program command response */
            /* Reponse to the 'Request to Finalise the Flash ROM sector'. Contains the 'Sector ID' of the 
               finalised sector (if successful) */
            ddProgramFinalise           = 0xA303,
            
            /* BST-BEM Read Program command response */
            /* Reponse to the 'Request to Read a Block of Flash ROM data'. Contains the 'Sector ID' of the 
               Flash ROM sector read from, the 32-bit Address offset in to that sector and the requested 
               data Block. Note that the 'Transfer Data Block (TDB)' size is the 'Data Payload' size - 6 
               bytes */
            ddProgramRead               = 0xA304,
            
            /* BST-BEM Negative Acknowledgement Program command response */
            /* Programming Negative Response from the device based on previous requests of it. Not normally 
               used */
            ddProgramNegAcknowledge     = 0xA3F4,
            
            
            /*---  BEM Program Protocol Commands ----------------------------------*/
            /*----------------------------------------------------------------------
              All commands sent from the PC using BEM Sub-ID code BST_BEMP_PRG_PG (0xA4),
              will use BEM sub-ID code BST_BEMP_MSG_GP (0xA3) for all required responses
              (ACKnowledge / NACKnowldege or Information), plus the BEM ID command.
            -----------------------------------------------------------------------*/
            
            /*---  Non-specific sub-commands 0x00-0x1F  ---------------------------*/
            /*----------------------------------------------------------------------
              Common debug codes - should not be product or software specific
            -----------------------------------------------------------------------*/
            
            /* BST-BEM Set Sector Program command */
            /* Request to Set the Sector of Flash ROM (prior to erasing, writing or reading). This sub-command 
               must be sent prior to any other 'Program' sub-command - as it sets up required Flash ROM 
               sector that all subsequent operations will use as a base - without it, no other Programming 
               operations are possible */
            ddProgramSetSectorPG        = 0xA400,
            
            /* BST-BEM Erase Program command */
            /* Request to Erase current sector of Flash ROM (prior to reprogramming). The actual sector 
               that this operation is performed on is dependent on how the target has been instructed to 
               understand the 'Sector ID'. This allows each target to operate within its own Flash ROM limits 
               whilst still using the same group of reprogramming interface sub-commands */
            ddProgramErasePG            = 0xA401,
            
            /* BST-BEM Write & Verify Program command */
            /* Request to Write (and then Verify) a Block of data bytes (up to maximum size that target 
               set) to requested Flash ROM offset address inside the current Sector. The Write block size 
               that the microcontroller can cope with was returned to the PC software in the 'Set Sector' 
               response message */
            ddProgramWriteVerifyPG      = 0xA402,
            
            /* BST-BEM Finalise Program command */
            /* Request to Finalise/Complete Flash ROM reprogramming operation by giving hardware the CRC 
               and size of the entire Data Block transferred. These values will be used by the hardware 
               to generate its own CRC of the modified Flash ROM area, so it can be matched against the 
               given CRC and the corresponding response returned to the PC software */
            ddProgramFinalisePG         = 0xA403,
            
            /* BST-BEM Read Program command */
            /* Request to Read a block data bytes (up to the maximum size that the target set) from the 
               requested Flash ROM offset address inside the currently set Sector */
            ddProgramReadPG             = 0xA404,
            
            /* End of Enumeration */
            /* Last Enumeration - used by software to determine end of BST Defines list */
            ddLastEnum                  = 0xFFFD,
        };
    }
}