﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ActisenseComms.API;
using System.Reflection;
using System.Diagnostics;

namespace NMEA_Transport
{
    public class NMEATransport : ITransport
    {
        String _name { set; get; }
        int ActiHandle;
        NMEA2K.sMsg Msg = new NMEA2K.sMsg();
        Callback RxCallback;

        const Byte N2K_BROADCAST_ADDRESS = 255;
        const int ISO_REQUEST_ID = 59904;

//        const int N2K_PROP_FP     = 126720;	   // Proprietary fast packet 0x1EF00
//        const int N2K_PROP_CLIENT = 126720;	   // Fusion Client Proprietary 0x1EF00 (addressed)

        Byte audioDeviceAddress = 255;

        public PacketReceivedDelegate PacketReceived { get; set; }

        public NMEATransport()
        {
            InitializeSerialPort();
        }

        public void SetAudioAddress(byte address)
        {
            audioDeviceAddress = address;
        }

        public String GetName()
        {
            return _name;
        }

        public void SetName(String name)
        {
            _name = name;
        }

        public void SendPacket(byte[] datagram)
        {
            byte[] data = new byte[datagram.Length + 2];

            // Fusion Proprietry Manufacture codes 
            ushort x = 419;
            x += 0x1800;    // reserved bits
            x += (4 << 13);

            data[0] = (byte)(x & 0xFF);
            data[1] = (byte)((x >> 8) & 0xFF);

            Buffer.BlockCopy(datagram, 0, data, 2, datagram.Length);

            WritePacket(NMEACommon.N2K_PROP_CLIENT, 6,data, (byte)audioDeviceAddress);
        }

        public void SendComplex(byte[] datagram)
        {
            WritePacket(NMEACommon.N2K_COMMAND_GROUP_FUNCTION, 3, datagram, (byte)audioDeviceAddress);
        }

        public void BroadcastIsoRequest(UInt32 pgn)
        {
            SendIsoRequest(pgn, NMEACommon.N2K_BROADCAST_ADDRESS);
        }

        public void SendIsoRequest(UInt32 pgn)
        {
            SendIsoRequest(pgn, (byte)audioDeviceAddress);
        }

        private void SendIsoRequest(UInt32 pgn, byte destAddress)
        {
            byte[] msg = new byte[3];

            msg[0] = (byte)(pgn & 0xFF);
            msg[1] = (byte)((pgn >> 8) & 0xFF);
            msg[2] = (byte)((pgn >> 16) & 0xFF);

            WritePacket(ISO_REQUEST_ID, 6, msg.ToArray(), destAddress);
        }

        public void SendComplexCommand(UInt32 pgn, byte[] datagram)
        {
            int Error;
            NMEA2K.sMsg txMsg = new NMEA2K.sMsg();

            txMsg.PGN = NMEACommon.N2K_COMMAND_GROUP_FUNCTION;
            txMsg.Priority = 3;
            txMsg.DestAddr = (byte)audioDeviceAddress;
            txMsg.Length = Convert.ToUInt32(datagram.Length + 5);

            unsafe
            {
                txMsg.Data[0] = 0x01;
                txMsg.Data[1] = (byte)(pgn & 0xFF);							// Field 2
                txMsg.Data[2] = (byte)((pgn >> 8) & 0xFF);					// Field 2
                txMsg.Data[3] = (byte)((pgn >> 16) & 0xFF);					// Field 2
                txMsg.Data[4] = (byte)8 ; // use same priority    // Field 3
                txMsg.Data[4] |= 0xF0; // Reserved bits					   Field 4
                unsafe
                {
                    int offset = 5;
                    for (int i = 0; i < datagram.Length; i++)
                    {
                        txMsg.Data[i + offset] = datagram[i];
                    }
                }
            }

            Error = NMEA2K.Write(ActiHandle, out txMsg);
            if (Error < 0) throw new Exception("Error Writing Packet");
        }

        public void SendComplexRequest(UInt32 pgn, byte[] datagram)
        {
            int Error;
            NMEA2K.sMsg txMsg = new NMEA2K.sMsg();

            txMsg.PGN = NMEACommon.N2K_COMMAND_GROUP_FUNCTION;
            txMsg.Priority = 3;
            txMsg.DestAddr = (byte)audioDeviceAddress;
            txMsg.Length = Convert.ToUInt32(datagram.Length + 10);

            unsafe
            {
                txMsg.Data[0] = 0x00;
                txMsg.Data[1] = (byte)(pgn & 0xFF);							// Field 2
                txMsg.Data[2] = (byte)((pgn >> 8) & 0xFF);					// Field 2
                txMsg.Data[3] = (byte)((pgn >> 16) & 0xFF);					// Field 2
                txMsg.Data[4] = 0xFF;                                       // Field 3 - 0xFFFF FFFF = Do not change interval
                txMsg.Data[5] = 0xFF;                                       // Field 3
                txMsg.Data[6] = 0xFF;                                       // Field 3
                txMsg.Data[7] = 0xFF;                                       // Field 3
                txMsg.Data[8] = 0xFF;                                       // Field 4 - 0xFFFF = Do not change offset
                txMsg.Data[9] = 0xFF;                                       // Field 4
                unsafe
                {
                    int offset = 10;
                    for (int i = 0; i < datagram.Length; i++)
                    {
                        txMsg.Data[i + offset] = datagram[i];
                    }
                }
            }

            Error = NMEA2K.Write(ActiHandle, out txMsg);
            if (Error < 0) throw new Exception("Error Writing Packet");
        }

        private void WritePacket(uint PGN, byte priority, byte[] datagram, byte destAddress)
        {
            int Error;
            NMEA2K.sMsg txMsg = new NMEA2K.sMsg();

            txMsg.PGN = PGN;
            txMsg.Priority = priority;
            txMsg.DestAddr = destAddress;
            txMsg.Length = Convert.ToUInt32(datagram.Length);

            unsafe
            {
                for (int i = 0; i < datagram.Length; i++)
                {
                    txMsg.Data[i] = datagram[i];
                }
            }

            Error = NMEA2K.Write(ActiHandle, out txMsg);
            if (Error < 0) throw new Exception("Error Writing Packet");
        }

        protected void InitializeSerialPort()
        {
            int Error;
            bool n2kportfound = false;
            /* Populate 'Serial Port Info' selection object on GUI with port friendly names */
            AComms.sPortEnum portEnum = new AComms.sPortEnum();
            Error = AComms.EnumerateSerialPorts(out portEnum);

            if (Error < 0) throw new Exception("Failed to retrieve serial port enumerations");
            for (uint i = 0; i < portEnum.Size; i++)
            {
                string name = AComms.EnumerateSerialPortsGetName(portEnum.Ports[i]);
                //Debug.WriteLine("Port found: " + name);

                if (name.Contains("Actisense"))
                {
                   // Debug.WriteLine("Found NMEA2000 Port");

                    /* API Create call creates a new instance of the ActisenseComms library */
                    Error = AComms.Create(out ActiHandle);
                    if (Error < 0) throw new Exception("Failed to initialize Actisense Comms Library");
                    /* Create the delegate object */
                    //Log.Enable(ActiHandle, "Amber", Log.Mask.COMMSLOG_EBL_TX | Log.Mask.COMMSLOG_EBL_RX);

                    RxCallback = new Callback(new Callback(this.RxCallbackHandler));

                    /* API SetRxCallback registers callback for incoming N2K messages with 
                     * the newly created Actisence Comms library instance */
                    Error = NMEA2K.SetRxCallback(ActiHandle, RxCallback, Missing.Value);
                    
                        
                    if (Error < 0) throw new Exception("Failed to setup requested callback handling function");

                    Error = AComms.Open(ActiHandle, (int)portEnum.Ports[i], 115200);
                    ACommand.SetOperatingMode(ActiHandle, 2);

                    n2kportfound = true;
                    break;
                }
                
            }
            if (!n2kportfound)
                throw new Exception("N2KNOPORT no n2k port found");
        }

        public byte myAddress()
        {
            Byte myAddress = 255;

            ACommand.GetN2KAddress(ActiHandle, out myAddress);

            return (myAddress);
        }

        /* -----------------------------------------------------------------
         * RxCallbackHandler method is called by ActisenseComms Rx message
         * handler every time a new NMEA 2000 message is sucessfully received
         * ----------------------------------------------------------------- */
        void RxCallbackHandler(Missing UserData)
        {
            int Error;

            /* NMEA2K.Read calls the ActisenseComms API to obtain the new N2K
             * message that has just been received */
            Error = NMEA2K.Read(ActiHandle, out Msg);
            if (Error < 0) throw new Exception("Failed to Read message from serial port");
            else
            {
                NMEA_Packet msgInfo = new NMEA_Packet(Msg);

                // Debug.WriteLine(msgInfo.ToString());

                if(msgInfo.PGN == NMEACommon.N2K_SUPPORTED_SOURCE_DATA)
                {
                    Debug.WriteLine(msgInfo.ToString());
                }

                PacketReceived(msgInfo);
            }
        }

    }
}
