﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NMEA_Transport;

namespace NMEA_Transport
{
    public class NMEA_Device
    {
        public Byte Address;
        public UInt16 NMEA_Version;
        public UInt16 ProductID;
        public String Model;
        public String SWVersion;
        public String ModelVersion;
        public String SerialNumber;
        public Byte Compliance;
        public Byte Load;

        public NMEA_Device(NMEA_Packet packet)
        {
            Address = packet.src_address;
            NMEA_Version = GetUInt16(packet.data, 0);
            ProductID = GetUInt16(packet.data, 2);
            Model = Encoding.UTF8.GetString(packet.data, 4, 32);
            SWVersion = Encoding.UTF8.GetString(packet.data, 36, 32);
            ModelVersion = Encoding.UTF8.GetString(packet.data, 68, 32);
            SerialNumber = Encoding.UTF8.GetString(packet.data, 100, 32);
            Compliance = packet.data[132];
            Load = packet.data[133];
        }

        protected static bool GetBool(byte[] datagram, int startIndex)
        {
            return datagram.Length > startIndex && datagram[startIndex] == 1;
        }

        /// <summary>
        /// Converts from network to host order, also takes into account
        /// the machines endian byte order.
        /// </summary>
        /// <param name="datagram">The upd datagram</param>
        /// <param name="startIndex">The starting index of UInt16 in question</param>
        protected static UInt16 GetUInt16(byte[] datagram, int startIndex)
        {
            ushort result;

            if (BitConverter.IsLittleEndian)
            {
                //LSB
                result = datagram[startIndex];
                result += (ushort)(datagram[startIndex + 1] << 8);
            }
            else
            {
                //MSB
                result = (ushort)(datagram[startIndex] << 8);
                result += datagram[startIndex + 1];
            }

            return result;
        }

        /// <summary>
        /// Converts from network to host order, method also takes into account
        /// the machines endian byte order.
        /// </summary>
        /// <param name="datagram">The upd datagram</param>
        /// <param name="startIndex">The starting index of UInt32 in question</param>
        /// <returns></returns>
        protected static UInt32 GetUInt32(byte[] datagram, int startIndex)
        {
            UInt32 result;

            if (BitConverter.IsLittleEndian)
            {
                //LSB
                result = datagram[startIndex];
                result += (UInt32)(datagram[startIndex + 1] << 8);
                result += (UInt32)(datagram[startIndex + 2] << 16);
                result += (UInt32)(datagram[startIndex + 3] << 24);
            }
            else
            {
                //MSB
                result = (UInt32)(datagram[startIndex] << 24);
                result += (UInt32)(datagram[startIndex + 1] << 16);
                result += (UInt32)(datagram[startIndex + 2] << 8);
                result += datagram[startIndex + 3];
            }

            return result;
        }
    }
}
